@@ -30,13 +30,13 @@
 """Read from and write to tar format archives.
 """
 
-__version__ = "$Revision: 37675 $"
+__version__ = "$Revision: 41341 $"
 # $Source$
 
 version     = "0.6.4"
 __author__  = "Lars Gustbel (lars@gustaebel.de)"
-__date__    = "$Date: 2004-10-20 21:49:28 +1000 (Wed, 20 Oct 2004) $"
-__cvsid__   = "$Id: tarfile.py 37675 2004-10-20 11:49:28Z akuchling $"
+__date__    = "$Date: 2005-10-28 08:00:51 +0200 (Fri, 28 Oct 2005) $"
+__cvsid__   = "$Id: tarfile.py 41341 2005-10-28 06:00:51Z neal.norwitz $"
 __credits__ = "Gustavo Niemeyer, Niels Gustbel, Richard Townsend."
 
 #---------
@@ -135,7 +135,7 @@ TOEXEC  = 0001           # execute/search by other
 def nts(s):
     """Convert a null-terminated string buffer to a python string.
     """
-    return s.split(NUL, 1)[0]
+    return s.rstrip(NUL)
 
 def calc_chksum(buf):
     """Calculate the checksum for a member's header. It's a simple addition
@@ -654,21 +654,27 @@ class TarInfo(object):
         self.offset_data = 0       # the file's data starts here
 
     def __repr__(self):
-        # On some systems (RH10) id() can be a negative number. 
-        # work around this.
-        MAX = 2L*sys.maxint+1
-        return "<%s %r at %#x>" % (self.__class__.__name__,self.name,
-                                   id(self)&MAX)
+        return "<%s %r at %#x>" % (self.__class__.__name__,self.name,id(self))
 
     def frombuf(cls, buf):
         """Construct a TarInfo object from a 512 byte string buffer.
         """
         tarinfo = cls()
-        tarinfo.name   =  nts(buf[0:100])
+        tarinfo.name   = nts(buf[0:100])
         tarinfo.mode   = int(buf[100:108], 8)
         tarinfo.uid    = int(buf[108:116],8)
         tarinfo.gid    = int(buf[116:124],8)
-        tarinfo.size   = long(buf[124:136], 8)
+
+        # There are two possible codings for the size field we
+        # have to discriminate, see comment in tobuf() below.
+        if buf[124] != chr(0200):
+            tarinfo.size = long(buf[124:136], 8)
+        else:
+            tarinfo.size = 0L
+            for i in range(11):
+                tarinfo.size <<= 8
+                tarinfo.size += ord(buf[125 + i])
+
         tarinfo.mtime  = long(buf[136:148], 8)
         tarinfo.chksum = int(buf[148:156], 8)
         tarinfo.type   = buf[156:157]
@@ -682,6 +688,11 @@ class TarInfo(object):
             tarinfo.devmajor = tarinfo.devmajor = 0
         tarinfo.prefix = buf[345:500]
 
+        # Some old tar programs represent a directory as a regular
+        # file with a trailing slash.
+        if tarinfo.isreg() and tarinfo.name.endswith("/"):
+            tarinfo.type = DIRTYPE
+
         # The prefix field is used for filenames > 100 in
         # the POSIX standard.
         # name = prefix + '/' + name
@@ -689,7 +700,7 @@ class TarInfo(object):
             tarinfo.name = normpath(os.path.join(nts(tarinfo.prefix), tarinfo.name))
 
         # Directory names should have a '/' at the end.
-        if tarinfo.isdir() and tarinfo.name[-1:] != "/":
+        if tarinfo.isdir():
             tarinfo.name += "/"
         return tarinfo
 
@@ -698,16 +709,28 @@ class TarInfo(object):
     def tobuf(self):
         """Return a tar header block as a 512 byte string.
         """
-        name = self.name
+        # Prefer the size to be encoded as 11 octal ascii digits
+        # which is the most portable. If the size exceeds this
+        # limit (>= 8 GB), encode it as an 88-bit value which is
+        # a GNU tar feature.
+        if self.size <= MAXSIZE_MEMBER:
+            size = "%011o" % self.size
+        else:
+            s = self.size
+            size = ""
+            for i in range(11):
+                size = chr(s & 0377) + size
+                s >>= 8
+            size = chr(0200) + size
 
         # The following code was contributed by Detlef Lannert.
         parts = []
         for value, fieldsize in (
-                (name, 100),
+                (self.name, 100),
                 ("%07o" % (self.mode & 07777), 8),
                 ("%07o" % self.uid, 8),
                 ("%07o" % self.gid, 8),
-                ("%011o" % self.size, 12),
+                (size, 12),
                 ("%011o" % self.mtime, 12),
                 ("        ", 8),
                 (self.type, 1),
@@ -721,7 +744,7 @@ class TarInfo(object):
                 (self.prefix, 155)
             ):
             l = len(value)
-            parts.append(value + (fieldsize - l) * NUL)
+            parts.append(value[:fieldsize] + (fieldsize - l) * NUL)
 
         buf = "".join(parts)
         chksum = calc_chksum(buf)
@@ -768,7 +791,7 @@ class TarFile(object):
                                 # messages (if debug >= 0). If > 0, errors
                                 # are passed to the caller as exceptions.
 
-    posix = True                # If True, generates POSIX.1-1990-compliant
+    posix = False               # If True, generates POSIX.1-1990-compliant
                                 # archives (no GNU extensions!)
 
     fileobject = ExFileObject
@@ -803,8 +826,6 @@ class TarFile(object):
         # Init datastructures
         self.closed      = False
         self.members     = []       # list of members as TarInfo objects
-        self.membernames = []       # names of members
-        self.chunks      = [0]      # chunk cache
         self._loaded     = False    # flag if all members have been read
         self.offset      = 0L       # current position in the archive file
         self.inodes      = {}       # dictionary caching the inodes of
@@ -1024,12 +1045,10 @@ class TarFile(object):
            than once in the archive, its last occurence is assumed to be the
            most up-to-date version.
         """
-        self._check()
-        if name not in self.membernames and not self._loaded:
-            self._load()
-        if name not in self.membernames:
+        tarinfo = self._getmember(name)
+        if tarinfo is None:
             raise KeyError, "filename %r not found" % name
-        return self._getmember(name)
+        return tarinfo
 
     def getmembers(self):
         """Return the members of the archive as a list of TarInfo objects. The
@@ -1045,10 +1064,7 @@ class TarFile(object):
         """Return the members of the archive as a list of their names. It has
            the same order as the list returned by getmembers().
         """
-        self._check()
-        if not self._loaded:
-            self._load()
-        return self.membernames
+        return [tarinfo.name for tarinfo in self.getmembers()]
 
     def gettarinfo(self, name=None, arcname=None, fileobj=None):
         """Create a TarInfo object for either the file `name' or the file
@@ -1092,7 +1108,8 @@ class TarFile(object):
         stmd = statres.st_mode
         if stat.S_ISREG(stmd):
             inode = (statres.st_ino, statres.st_dev)
-            if inode in self.inodes and not self.dereference:
+            if not self.dereference and \
+                    statres.st_nlink > 1 and inode in self.inodes:
                 # Is it a hardlink to an already
                 # archived file?
                 type = LNKTYPE
@@ -1121,13 +1138,16 @@ class TarFile(object):
 
         # Fill the TarInfo object with all
         # information we can get.
-        tarinfo.name  = arcname
-        tarinfo.mode  = stmd
-        tarinfo.uid   = statres.st_uid
-        tarinfo.gid   = statres.st_gid
-        tarinfo.size  = statres.st_size
+        tarinfo.name = arcname
+        tarinfo.mode = stmd
+        tarinfo.uid = statres.st_uid
+        tarinfo.gid = statres.st_gid
+        if stat.S_ISREG(stmd):
+            tarinfo.size = statres.st_size
+        else:
+            tarinfo.size = 0L
         tarinfo.mtime = statres.st_mtime
-        tarinfo.type  = type
+        tarinfo.type = type
         tarinfo.linkname = linkname
         if pwd:
             try:
@@ -1218,16 +1238,15 @@ class TarFile(object):
             self.addfile(tarinfo, f)
             f.close()
 
-        if tarinfo.type in (LNKTYPE, SYMTYPE, FIFOTYPE, CHRTYPE, BLKTYPE):
-            tarinfo.size = 0L
-            self.addfile(tarinfo)
-
-        if tarinfo.isdir():
+        elif tarinfo.isdir():
             self.addfile(tarinfo)
             if recursive:
                 for f in os.listdir(name):
                     self.add(os.path.join(name, f), os.path.join(arcname, f))
 
+        else:
+            self.addfile(tarinfo)
+
     def addfile(self, tarinfo, fileobj=None):
         """Add the TarInfo object `tarinfo' to the archive. If `fileobj' is
            given, tarinfo.size bytes are read from it and added to the archive.
@@ -1246,7 +1265,11 @@ class TarFile(object):
             tarinfo.linkname = normpath(tarinfo.linkname)
 
         if tarinfo.size > MAXSIZE_MEMBER:
-            raise ValueError, "file is too large (>8GB)"
+            if self.posix:
+                raise ValueError, "file is too large (>= 8 GB)"
+            else:
+                self._dbg(2, "tarfile: Created GNU tar largefile header")
+
 
         if len(tarinfo.linkname) > LENGTH_LINK:
             if self.posix:
@@ -1290,8 +1313,6 @@ class TarFile(object):
             self.offset += blocks * BLOCKSIZE
 
         self.members.append(tarinfo)
-        self.membernames.append(tarinfo.name)
-        self.chunks.append(self.offset)
 
     def extract(self, member, path=""):
         """Extract a member from the archive to the current working directory,
@@ -1306,6 +1327,10 @@ class TarFile(object):
         else:
             tarinfo = self.getmember(member)
 
+        # Prepare the link target for makelink().
+        if tarinfo.islnk():
+            tarinfo._link_target = os.path.join(path, tarinfo.linkname)
+
         try:
             self._extract_member(tarinfo, os.path.join(path, tarinfo.name))
         except EnvironmentError, e:
@@ -1353,7 +1378,7 @@ class TarFile(object):
                 # stream of tar blocks.
                 raise StreamError, "cannot extract (sym)link as file object"
             else:
-                # A (sym)link's file object is it's target's file object.
+                # A (sym)link's file object is its target's file object.
                 return self.extractfile(self._getmember(tarinfo.linkname,
                                                         tarinfo))
         else:
@@ -1478,7 +1503,8 @@ class TarFile(object):
             if tarinfo.issym():
                 os.symlink(linkpath, targetpath)
             else:
-                os.link(linkpath, targetpath)
+                # See extract().
+                os.link(tarinfo._link_target, targetpath)
         except AttributeError:
             if tarinfo.issym():
                 linkpath = os.path.join(os.path.dirname(tarinfo.name),
@@ -1559,7 +1585,7 @@ class TarFile(object):
             return m
 
         # Read the next block.
-        self.fileobj.seek(self.chunks[-1])
+        self.fileobj.seek(self.offset)
         while True:
             buf = self.fileobj.read(BLOCKSIZE)
             if not buf:
@@ -1577,7 +1603,7 @@ class TarFile(object):
                     continue
                 else:
                     # Block is empty or unreadable.
-                    if self.chunks[-1] == 0:
+                    if self.offset == 0:
                         # If the first block is invalid. That does not
                         # look like a tar archive we can handle.
                         raise ReadError,"empty, unreadable or compressed file"
@@ -1600,20 +1626,14 @@ class TarFile(object):
         # Check if the TarInfo object has a typeflag for which a callback
         # method is registered in the TYPE_METH. If so, then call it.
         if tarinfo.type in self.TYPE_METH:
-            tarinfo = self.TYPE_METH[tarinfo.type](self, tarinfo)
-        else:
-            tarinfo.offset_data = self.offset
-            if tarinfo.isreg() or tarinfo.type not in SUPPORTED_TYPES:
-                # Skip the following data blocks.
-                self.offset += self._block(tarinfo.size)
+            return self.TYPE_METH[tarinfo.type](self, tarinfo)
 
-        if tarinfo.isreg() and tarinfo.name[:-1] == "/":
-            # some old tar programs don't know DIRTYPE
-            tarinfo.type = DIRTYPE
+        tarinfo.offset_data = self.offset
+        if tarinfo.isreg() or tarinfo.type not in SUPPORTED_TYPES:
+            # Skip the following data blocks.
+            self.offset += self._block(tarinfo.size)
 
         self.members.append(tarinfo)
-        self.membernames.append(tarinfo.name)
-        self.chunks.append(self.offset)
         return tarinfo
 
     #--------------------------------------------------------------------------
@@ -1628,15 +1648,15 @@ class TarFile(object):
     #    if there is data to follow.
     # 2. set self.offset to the position where the next member's header will
     #    begin.
-    # 3. return a valid TarInfo object.
+    # 3. append the tarinfo object to self.members, if it is supposed to appear
+    #    as a member of the TarFile object.
+    # 4. return tarinfo or another valid TarInfo object.
 
     def proc_gnulong(self, tarinfo):
         """Evaluate the blocks that hold a GNU longname
            or longlink member.
         """
         buf = ""
-        name = None
-        linkname = None
         count = tarinfo.size
         while count > 0:
             block = self.fileobj.read(BLOCKSIZE)
@@ -1644,24 +1664,16 @@ class TarFile(object):
             self.offset += BLOCKSIZE
             count -= BLOCKSIZE
 
+        # Fetch the next header
+        next = self.next()
+
+        next.offset = tarinfo.offset
         if tarinfo.type == GNUTYPE_LONGNAME:
-            name = nts(buf)
-        if tarinfo.type == GNUTYPE_LONGLINK:
-            linkname = nts(buf)
+            next.name = nts(buf)
+        elif tarinfo.type == GNUTYPE_LONGLINK:
+            next.linkname = nts(buf)
 
-        buf = self.fileobj.read(BLOCKSIZE)
-
-        tarinfo = TarInfo.frombuf(buf)
-        tarinfo.offset = self.offset
-        self.offset += BLOCKSIZE
-        tarinfo.offset_data = self.offset
-        tarinfo.name = name or tarinfo.name
-        tarinfo.linkname = linkname or tarinfo.linkname
-
-        if tarinfo.isreg() or tarinfo.type not in SUPPORTED_TYPES:
-            # Skip the following data blocks.
-            self.offset += self._block(tarinfo.size)
-        return tarinfo
+        return next
 
     def proc_sparse(self, tarinfo):
         """Analyze a GNU sparse header plus extra headers.
@@ -1717,6 +1729,8 @@ class TarFile(object):
         tarinfo.offset_data = self.offset
         self.offset += self._block(tarinfo.size)
         tarinfo.size = origsize
+
+        self.members.append(tarinfo)
         return tarinfo
 
     # The type mapping for the next() method. The keys are single character
@@ -1744,14 +1758,17 @@ class TarFile(object):
         """Find an archive member by name from bottom to top.
            If tarinfo is given, it is used as the starting point.
         """
+        # Ensure that all members have been loaded.
+        members = self.getmembers()
+
         if tarinfo is None:
-            end = len(self.members)
+            end = len(members)
         else:
-            end = self.members.index(tarinfo)
+            end = members.index(tarinfo)
 
         for i in xrange(end - 1, -1, -1):
-            if name == self.membernames[i]:
-                return self.members[i]
+            if name == members[i].name:
+                return members[i]
 
     def _load(self):
         """Read through the entire archive file and look for readable
@@ -1786,6 +1803,8 @@ class TarFile(object):
            of the longname as size, followed by data blocks,
            which contain the longname as a null terminated string.
         """
+        name += NUL
+
         tarinfo = TarInfo()
         tarinfo.name = "././@LongLink"
         tarinfo.type = type
@@ -1794,6 +1813,7 @@ class TarFile(object):
 
         # write extended header
         self.fileobj.write(tarinfo.tobuf())
+        self.offset += BLOCKSIZE
         # write name blocks
         self.fileobj.write(name)
         blocks, remainder = divmod(tarinfo.size, BLOCKSIZE)
@@ -1820,6 +1840,7 @@ class TarIter:
         """Construct a TarIter object.
         """
         self.tarfile = tarfile
+        self.index = 0
     def __iter__(self):
         """Return iterator object.
         """
@@ -1828,10 +1849,20 @@ class TarIter:
         """Return the next item using TarFile's next() method.
            When all members have been read, set TarFile as _loaded.
         """
-        tarinfo = self.tarfile.next()
-        if not tarinfo:
-            self.tarfile._loaded = True
-            raise StopIteration
+        # Fix for SF #1100429: Under rare circumstances it can
+        # happen that getmembers() is called during iteration,
+        # which will cause TarIter to stop prematurely.
+        if not self.tarfile._loaded:
+            tarinfo = self.tarfile.next()
+            if not tarinfo:
+                self.tarfile._loaded = True
+                raise StopIteration
+        else:
+            try:
+                tarinfo = self.tarfile.members[self.index]
+            except IndexError:
+                raise StopIteration
+        self.index += 1
         return tarinfo
 
 # Helper classes for sparse file support
